library in_level;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Level Input Module for Lightning.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2011
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

{$IMAGEBASE $410B0000}

uses
  Classes,
  SysUtils,
  Windows,
  Controls,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  Graphics,
  Dialogs,
  common in '..\common.pas';

{$R *.RES}
{$R ..\RESOURCES\OVERRIDE.RES}

type
  TDetails = class(TObject)
  public
    origin, aim: TLevel;
    fade:        Cardinal; // fade time
    lastUpdate:  Int64;
    pLevel:      TPanel;
    tbLevel:     TTrackBar;
    pOverride:   TPanel;
    iOverride:   TImage;
    overridden:  Boolean;
    procedure iOverrideMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    function  getLevel(): TLevel;
    procedure setLevel(level: TLevel);
  end;

procedure TDetails.iOverrideMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  overridden := not(overridden);
  if overridden then
    iOverride.Picture.Bitmap.LoadFromResourceName(hInstance, 'overridedown')
  else
    iOverride.Picture.Bitmap.LoadFromResourceName(hInstance, 'overrideup');
end;

function  TDetails.getLevel(): TLevel;
begin
  Result := levelMax - tbLevel.Position;
end;

procedure TDetails.setLevel(level: TLevel);
begin
  tbLevel.Position := levelMax - level;
end;

function init(name: PChar; nameLength: DWORD): DWORD; cdecl;
begin
  Result := return('ILevel|Controls', name, nameLength);
  tickInit();
end;

procedure finish(); cdecl;
begin
  //
end;

function create(controlHandle: THandle): PLightningModule; cdecl;
var
  myDetails: TDetails;
begin
  New(Result);
  with Result^ do
  begin
    handle        := controlHandle;
    title         := 'Level';
    settings      := False;
    width         := 48;
    height        := 168;
    inputTop      := -1;
    infoTop       := -1;
    outputTop     := 144;
    inputs        := 0;
    outputs       := 1;
    inputTooltip  := '';
    outputTooltip := 'Level value';
    myDetails     := TDetails.Create;
    reserved      := nil;
    details       := myDetails;
  end;
  with myDetails do
  begin
    fade       := 0;
    overridden := False;
    pOverride := TPanel.Create(nil);
    with pOverride do
    begin
      Top        := 162;
      Left       := 1;
      BevelOuter := bvNone;
    end;
    iOverride := TImage.Create(pOverride);
    with iOverride do
    begin
      Cursor      := crHandPoint;
      AutoSize    := True;
      Hint        := 'Manual override';
      ShowHint    := True;
      Picture.Bitmap.LoadFromResourceName(hInstance, 'overrideup');
      OnMouseDown := iOverrideMouseDown;
      Parent      := pOverride;
    end;
    with pOverride do
    begin
      Width        := iOverride.Width;
      Height       := iOverride.Height;
      ParentWindow := controlHandle;
    end;
    pLevel := TPanel.Create(nil);
    with pLevel do
    begin
      Top        := 24;
      Left       := 10;
      BevelOuter := bvNone;
    end;
    tbLevel := TTrackBar.Create(pLevel);
    with tbLevel do
    begin
      Orientation := trVertical;
      Width       := 30;
      Height      := 118;
      Frequency   := 300;
      Min         := levelMin;
      Max         := levelMax;
      Position    := levelMax;
      LineSize    := 1;
      PageSize    := 100;
      Parent      := pLevel;
    end;
    with pLevel do
    begin
      Width        := tbLevel.Width;
      Height       := tbLevel.Height;
      ParentWindow := controlHandle;
    end;
  end;
  New(PLightningModule2(Result^.reserved));
  PLightningModule2(Result^.reserved)^.reserved := nil;
end;

function  action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
var
  details:  TDetails;
  position: Integer;
  time:     Int64;
begin
  details := TDetails(lightningModule^.details);
  if not(PLightningModule2(lightningModule^.reserved)^.scenic) then
    details.pOverride.Visible := False;
  position := details.getLevel();
  if details.fade > 0 then
  begin
    if (position = details.aim) or details.overridden then
      details.fade := 0
    else
    begin
      time := tickCount;
      position :=
        position +
        Round(
          (time - details.lastUpdate) // shift multiplier (milliseconds)
          *
          ((details.aim - details.origin) / details.fade) // shift per millisecond
        );
      if position < levelMin then
        position := levelMin
      else if position > levelMax then
        position := levelMax;
      if position = details.aim then
        details.fade := 0
      else
        if ((details.origin < details.aim) and (position > details.aim))
           or
           ((details.origin > details.aim) and (position < details.aim))
           then
      begin
        position := details.aim;
        details.fade := 0;
      end;
      details.setLevel(position);
      details.lastUpdate := time;
    end;
  end;
  createSpectrum(bridge, details.getLevel());
  Result := True;
end;

procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
begin
  //
end;

procedure hide(lightningModule: PLightningModule); cdecl;
begin
  //
end;

procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
var
  bString: string;
begin
  bString := settings;
  with TDetails(lightningModule^.details) do
  begin
    setLevel(StrToIntDef(parse(bString, ','), levelMin));
    overridden := parse(bString, ',') = '';
    parse(bString, ','); // a label was stored here previously
    iOverrideMouseDown(nil, mbLeft, [], 0, 0);
  end;
end;

function  save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
var
  bString: string;
begin
  with TDetails(lightningModule^.details) do
  begin
    if overridden then
      bString := '1'
    else
      bString := '';
    Result := return(IntToStr(getLevel) + ',' + bString, settings, settingsLength)
  end;
end;

procedure openscene(lightningModule: PLightningModule; settings: PChar; fadeTime: Cardinal; reserved: Pointer); cdecl;
var
  bString: string;
begin
  bString := settings;
  with TDetails(lightningModule^.details) do
  begin
    if not(overridden) then
    begin
      lastUpdate := tickCount;
      fade       := fadeTime;
      origin     := getLevel;
      aim        := StrToInt(parse(bString, '|'));
      if fadeTime = 0 then
        setLevel(aim);
    end;
  end;
end;

function  savescene(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
begin
  with TDetails(lightningModule^.details) do
    Result := return(IntToStr(getLevel), settings, settingsLength);
end;

procedure destroy(lightningModule: PLightningModule); cdecl;
begin
  with TDetails(lightningModule^.details) do
  begin
    tbLevel.Free;
    pLevel.Free;
    iOverride.Free;
    pOverride.Free;
    Free;
  end;
  Dispose(PLightningModule2(lightningModule^.reserved));
  Dispose(lightningModule);
end;

exports
  init, finish, create, action, show, hide, open, save, openscene, savescene, destroy;

end.
